# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'curveselect.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_CurveSelection(object):
    def setupUi(self, CurveSelection):
        CurveSelection.setObjectName("CurveSelection")
        CurveSelection.resize(292, 329)
        self.verticalLayout = QtWidgets.QVBoxLayout(CurveSelection)
        self.verticalLayout.setObjectName("verticalLayout")
        self.lstCurves = QtWidgets.QListWidget(CurveSelection)
        self.lstCurves.setObjectName("lstCurves")
        self.verticalLayout.addWidget(self.lstCurves)
        self.btnProperties = QtWidgets.QPushButton(CurveSelection)
        self.btnProperties.setObjectName("btnProperties")
        self.verticalLayout.addWidget(self.btnProperties)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.okButton = QtWidgets.QPushButton(CurveSelection)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.okButton.sizePolicy().hasHeightForWidth())
        self.okButton.setSizePolicy(sizePolicy)
        self.okButton.setChecked(False)
        self.okButton.setObjectName("okButton")
        self.horizontalLayout.addWidget(self.okButton)
        self.cancelButton = QtWidgets.QPushButton(CurveSelection)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cancelButton.sizePolicy().hasHeightForWidth())
        self.cancelButton.setSizePolicy(sizePolicy)
        self.cancelButton.setObjectName("cancelButton")
        self.horizontalLayout.addWidget(self.cancelButton)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(CurveSelection)
        QtCore.QMetaObject.connectSlotsByName(CurveSelection)

    def retranslateUi(self, CurveSelection):
        _translate = QtCore.QCoreApplication.translate
        CurveSelection.setWindowTitle(_translate("CurveSelection", "Select visible curves"))
        self.btnProperties.setText(_translate("CurveSelection", "Curve Properties"))
        self.okButton.setText(_translate("CurveSelection", "Ok"))
        self.cancelButton.setText(_translate("CurveSelection", "Cancel"))



from typing import List

import pandas as pd
from graphysio.plotwidgets.curves import CurveItem

try:
    import pyarrow.parquet
except ImportError:
    is_available = False
else:
    is_available = True


def export_curves(
    curves: List[CurveItem], filepath: str, index_label: str = 'timens'
) -> None:
    sers = [c.series for c in curves]
    data = pd.concat(sers, axis=1).sort_index()
    data.to_parquet(
        filepath,
        engine='pyarrow',
        compression='gzip',
        coerce_timestamps='us',
        allow_truncated_timestamps=True,
    )


export_curves.is_available = is_available



from .csvrequest import Ui_NewPlot
from .curveproperties import Ui_CurveProperties
from .curveselect import Ui_CurveSelection
from .cycledetect import Ui_CycleDetection
from .datetime import Ui_SetDateTime
from .filter import Ui_Filter
from .loopwidget import Ui_LoopWidget
from .mainwindow import Ui_MainWindow
from .periodexport import Ui_PeriodExport
from .poiwidget import Ui_POISelectorWidget
from .setuppuloop import Ui_SetupPULoop



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'setuppuloop.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_SetupPULoop(object):
    def setupUi(self, SetupPULoop):
        SetupPULoop.setObjectName("SetupPULoop")
        SetupPULoop.resize(387, 371)
        self.verticalLayout = QtWidgets.QVBoxLayout(SetupPULoop)
        self.verticalLayout.setObjectName("verticalLayout")
        self.formLayout_2 = QtWidgets.QFormLayout()
        self.formLayout_2.setSizeConstraint(QtWidgets.QLayout.SetMaximumSize)
        self.formLayout_2.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout_2.setObjectName("formLayout_2")
        self.label = QtWidgets.QLabel(SetupPULoop)
        self.label.setObjectName("label")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label)
        self.comboU = QtWidgets.QComboBox(SetupPULoop)
        self.comboU.setObjectName("comboU")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.comboU)
        self.label_2 = QtWidgets.QLabel(SetupPULoop)
        self.label_2.setObjectName("label_2")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.comboP = QtWidgets.QComboBox(SetupPULoop)
        self.comboP.setObjectName("comboP")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.comboP)
        self.verticalLayout.addLayout(self.formLayout_2)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSizeConstraint(QtWidgets.QLayout.SetMinimumSize)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.okButton = QtWidgets.QPushButton(SetupPULoop)
        self.okButton.setObjectName("okButton")
        self.horizontalLayout.addWidget(self.okButton)
        self.cancelButton = QtWidgets.QPushButton(SetupPULoop)
        self.cancelButton.setObjectName("cancelButton")
        self.horizontalLayout.addWidget(self.cancelButton)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(SetupPULoop)
        QtCore.QMetaObject.connectSlotsByName(SetupPULoop)

    def retranslateUi(self, SetupPULoop):
        _translate = QtCore.QCoreApplication.translate
        SetupPULoop.setWindowTitle(_translate("SetupPULoop", "Set up PU-Loop"))
        self.label.setText(_translate("SetupPULoop", "Velocity (U)"))
        self.label_2.setText(_translate("SetupPULoop", "Pressure (P)"))
        self.okButton.setText(_translate("SetupPULoop", "Ok"))
        self.cancelButton.setText(_translate("SetupPULoop", "Cancel"))



from typing import List

import pandas as pd
from graphysio.plotwidgets.curves import CurveItem


def curves_to_csv(
    curves: List[CurveItem], filepath: str, index_label: str = 'timens'
) -> None:
    sers = [c.series for c in curves]
    data = pd.concat(sers, axis=1).sort_index()
    data['datetime'] = pd.to_datetime(data.index, unit='ns')
    data.to_csv(filepath, date_format="%Y-%m-%d %H:%M:%S.%f", index_label=index_label)


curves_to_csv.is_available = True



import itertools
import os
import sys
import traceback
from concurrent.futures import ThreadPoolExecutor
from functools import partial
from queue import Empty, Queue

from PyQt5 import QtCore, QtWidgets

from graphysio import dialogs, readdata, ui, utils
from graphysio.plotwidgets import TimeAxisItem, TSWidget


class MainUi(ui.Ui_MainWindow, QtWidgets.QMainWindow):
    setcoords = QtCore.pyqtSignal(float, float)

    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self.setupUi(self)
        self.dircache = os.path.expanduser('~')

        self.dataq = Queue()
        self.executor = ThreadPoolExecutor()
        self.datahandler = self.createNewPlotWithData

        self.tabWidget.tabCloseRequested.connect(self.closeTab)
        self.tabWidget.currentChanged.connect(self.tabChanged)

        getCLIShell = partial(utils.getshell, ui=self)

        launchNewPlot = partial(self.launchOpenFile, self.createNewPlotWithData)
        launchAppendPlot = partial(self.launchOpenFile, self.appendToPlotWithData)
        self.menuFile.addAction('New Plot', launchNewPlot)
        self.menuFile.addAction('Append to Plot', launchAppendPlot)

        self.menuFile.addSeparator()
        self.menuFile.addAction('&Load plugin', self.errguard(utils.loadmodule))
        self.menuFile.addAction('Get CLI shell', self.errguard(getCLIShell))
        self.menuFile.addSeparator()
        self.menuFile.addAction('&Quit', self.close, QtCore.Qt.CTRL + QtCore.Qt.Key_Q)

        self.setcoords.connect(self.setCoords)

        # Launch timer to handle new plot data
        self.data_timer = QtCore.QTimer()
        self.data_timer.timeout.connect(self.read_plot_data)
        self.data_timer.setInterval(1000)  # Milliseconds
        self.data_timer.start()

    def setCoords(self, x, y):
        if TimeAxisItem.is_relative_time(x):
            timestr = TimeAxisItem.conv_relative(x)
        else:
            timestr = TimeAxisItem.conv_absolute(x, mainwindow=True)
        self.lblCoords.setText(f'Time: {timestr}, Value: {y}')

    def read_plot_data(self):
        try:
            data = self.dataq.get(block=False)
        except Empty:
            return
        # Plotdata can be an object or a list of objects
        try:
            iter(data)
        except TypeError:
            data = [data]
        for plotdata in data:
            self.datahandler(plotdata)

    def print_exception(self, e):
        traceback.print_exc(file=sys.stdout)
        utils.displayError(e)

    def errguard(self, f):
        # Lift exceptions to UI reported errors
        def wrapped():
            try:
                f()
            except Exception as e:
                self.print_exception(e)

        return wrapped

    def addTab(self, widget: QtWidgets.QWidget, tabname: str) -> None:
        tabnames = []
        for i in range(self.tabWidget.count()):
            tabnames.append(self.tabWidget.tabText(i))
        if tabname in tabnames:
            tmptabname = tabname
            # Duplicate tab name. Add a number to the end
            for i in itertools.count():
                tmptabname = f'{tabname}{i+1}'
                if tmptabname not in tabnames:
                    break
            tabname = tmptabname
        tabindex = self.tabWidget.addTab(widget, tabname)
        self.tabWidget.setCurrentIndex(tabindex)

    def closeTab(self, i) -> None:
        w = self.tabWidget.widget(i)
        self.tabWidget.removeTab(i)
        w.close()
        w.deleteLater()
        del w

    def tabChanged(self, tabid):
        destwidget = self.tabWidget.widget(tabid)
        if destwidget is None:
            return
        self.menuBar.clear()
        self.menuBar.addMenu(self.menuFile)
        for title, submenu in destwidget.menu.items():
            menu = self.menuBar.addMenu(title)
            for title, item in submenu.items():
                menu.addAction(title, item)

    def launchOpenFile(self, datahandler):
        reader = readdata.FileReader()
        self.dircache = reader.askFile(self.dircache)
        future = self.executor.submit(reader.get_plotdata)

        def cb(future):
            plotdata = future.result()
            if plotdata:
                self.dataq.put(plotdata)

        self.datahandler = datahandler
        future.add_done_callback(cb)

    def createNewPlotWithData(self, plotdata):
        properties = {'dircache': self.dircache}
        plotwidget = TSWidget(plotdata, parent=self, properties=properties)
        self.addTab(plotwidget, plotdata.name)
        self.lblStatus.setText("Loading... done")

    def appendToPlotWithData(self, plotdata, destidx=None):
        if destidx is None:
            plotwidget = self.tabWidget.currentWidget()
        else:
            plotwidget = self.tabWidget.widget(destidx)
        if plotwidget is None:
            utils.displayError('No plot selected.')
            return
        dorealign = dialogs.userConfirm(
            'Timeshift new curves to make the beginnings coincide?',
            title='Append to plot',
        )

        for fieldname in plotdata.data:
            newname = plotwidget.validateNewCurveName(fieldname)
            if newname is None:
                continue
            if newname != fieldname:
                plotdata.data.rename(columns={fieldname: newname}, inplace=True)

        plotwidget.appendData(plotdata, dorealign)
        plotwidget.properties['dircache'] = self.dircache
        self.lblStatus.setText("Loading... done")



import pyqtgraph as pg


class LegendItem(pg.LegendItem):
    def __init__(self, size=None, offset=(40, 5)):
        super().__init__(size, offset)

    def paint(self, p, *args):
        p.setPen(pg.mkPen(0, 0, 0, 255))
        p.setBrush(pg.mkBrush(255, 255, 255, 255))
        p.drawRect(self.boundingRect())

    def addItem(self, item, name):
        # Fix the line width bigger
        pen = pg.mkPen(item.opts['pen'])
        pen.setWidth(2)
        newitem = pg.PlotDataItem(pen=pen)
        super().addItem(newitem, name)



from datetime import datetime
from math import floor
from typing import Dict

import numexpr as ne
import numpy as np
import pandas as pd
from graphysio.structures import Filter, Parameter
from graphysio.utils import truncatevecs
from pint import UnitRegistry
from scipy import interpolate, signal


class TF(object):
    def __init__(self, num, den, name=''):
        self.num = num
        self.den = den
        self.name = name

    def discretize(self, samplerate):
        systf = (self.num, self.den)
        (dnum, dden, _) = signal.cont2discrete(systf, 1 / samplerate)
        return (np.squeeze(dnum), np.squeeze(dden))


interpkind = ['linear', 'zero', 'pchip', 'nearest']
Filters = {
    'Lowpass filter': Filter(
        name='lowpass',
        parameters=[
            Parameter('Cutoff frequency (Hz)', int),
            Parameter('Filter order', int),
        ],
    ),
    'Filter ventilation': Filter(name='ventilation', parameters=[]),
    'Savitzky-Golay': Filter(
        name='savgol',
        parameters=[
            Parameter('Window duration', 'time'),
            Parameter('Polynomial order', int),
        ],
    ),
    'Interpolate': Filter(
        name='interp',
        parameters=[
            Parameter('New sampling rate (Hz)', float),
            Parameter('Interpolation type', interpkind),
        ],
    ),
    'Doppler cut': Filter(
        name='dopplercut', parameters=[Parameter('Minimum value', int)]
    ),
    'Differentiate': Filter(
        name='diff',
        parameters=[Parameter('Order', int), Parameter('Denominator time unit', str)],
    ),
    'Integrate': Filter(
        name='integrate', parameters=[Parameter('Window duration', 'time')]
    ),
    'Lag': Filter(name='lag', parameters=[Parameter('Time delta', 'time')]),
    'Normalize': Filter(name='norm1', parameters=[]),
    'Set start date/time': Filter(
        name='setdatetime', parameters=[Parameter('DateTime', datetime)]
    ),
    'Tolerant Normalize': Filter(name='norm2', parameters=[]),
    'Enter expression (variable = x)': Filter(
        name='expression', parameters=[Parameter('Expression', str)]
    ),
    'Strided Moving average': Filter(
        name='sma', parameters=[Parameter('Window duration', 'time')]
    ),
}

FeetFilters = {
    'Short cycles': Filter(
        name='shortcycles', parameters=[Parameter('Minimum duration', 'time')]
    )
}

TFs: Dict[str, TF] = {}


def updateTFs():
    tflist = list(TFs.keys())
    if not tflist:
        return
    Filters['Transfer function'] = Filter(
        name='tf', parameters=[Parameter('Transfer function', tflist)]
    )


def norm1(series, samplerate, parameters):
    series -= np.mean(series)
    series /= np.max(series) - np.min(series)
    newname = f'{series.name}-norm'
    return (series.rename(newname), None)


def norm2(series, samplerate, parameters):
    series -= np.mean(series)
    series /= np.std(series)
    newname = f'{series.name}-norm'
    return (series.rename(newname), None)


def expression(series, samplerate, parameters):
    (express,) = parameters
    try:
        filtered = ne.evaluate(express, local_dict={'x': series.to_numpy()})
    except Exception:
        filtered = None
    newname = f'{series.name}-filtered'
    newseries = pd.Series(filtered, index=series.index, name=newname)
    return (newseries, None)


def setdatetime(series, samplerate, parameters):
    (timestamp,) = parameters
    newseries = series.copy()
    diff = timestamp - newseries.index[0]
    newseries.index += diff
    newseries = newseries.rename(f'{series.name}-{timestamp}')
    return (newseries, None)


def sma(series, samplerate, parameters):
    (window_s,) = parameters
    serarr = series.to_numpy()
    serlen = len(serarr)
    winsize = int(window_s * samplerate)
    nwindows = floor(serlen / winsize)
    valstarts = np.arange(0, serlen - winsize, step=winsize)
    resiter = (
        np.mean(serarr[beg:end]) for beg, end in zip(valstarts, valstarts + winsize)
    )
    result = np.fromiter(resiter, dtype=np.float64, count=nwindows)
    locidx = winsize * np.arange(0, nwindows) + winsize // 2
    newname = f'{series.name}-sma{window_s}s'
    newseries = pd.Series(result, index=series.index[locidx], name=newname)
    newsamplerate = samplerate / winsize
    return (newseries, newsamplerate)


def savgol(series, samplerate, parameters):
    window, order = parameters
    window = np.floor(window * samplerate)
    if not window % 2:
        # window is even, we need odd
        window += 1
    filtered = signal.savgol_filter(series.values, int(window), order)
    newname = f'{series.name}-filtered'
    newseries = pd.Series(filtered, index=series.index, name=newname)
    return (newseries, None)


def lag(series, samplerate, parameters):
    (timedelta,) = parameters
    nindex = series.index.values - timedelta * 1e9
    newname = f'{series.name}-{timedelta}s'
    newseries = pd.Series(series.values, index=nindex, name=newname)
    return (newseries, None)


def tf(series, samplerate, parameters):
    (filtname,) = parameters
    tf = TFs[filtname]
    b, a = tf.discretize(samplerate)
    filtered = signal.lfilter(b, a, series)
    newname = f'{series.name}-{tf.name}'
    newseries = pd.Series(filtered, index=series.index, name=newname)
    return (newseries, None)


def lowpass(series, samplerate, parameters):
    Fc, order = parameters
    Wn = Fc * 2 / samplerate
    b, a = signal.butter(order, Wn)
    filtered = signal.lfilter(b, a, series)
    newname = f'{series.name}-lp{Fc}'
    newseries = pd.Series(filtered, index=series.index, name=newname)
    return (newseries, None)


def ventilation(series, samplerate, parameters):
    order = 12
    # Filter between 10 and 20 per minute
    bornes_hz = np.array([10, 20]) / 60
    Wn = [Fc * 2 / samplerate for Fc in bornes_hz]
    b, a = signal.butter(order, Wn, btype='bandstop')
    filtered = signal.filtfilt(b, a, series.values)
    newname = f'{series.name}-novent'
    newseries = pd.Series(filtered, index=series.index, name=newname)
    return (newseries, None)


def interp(series, samplerate, parameters):
    newsamplerate, method = parameters
    oldidx = series.index
    if method == 'pchip':
        f = interpolate.PchipInterpolator(oldidx, series.values, extrapolate=True)
    else:
        f = interpolate.interp1d(
            oldidx, series.values, kind=method, fill_value='extrapolate'
        )
    step = 1e9 / newsamplerate  # 1e9 to convert Hz to ns
    newidx = np.arange(oldidx[0], oldidx[-1], step, dtype=np.int64)
    resampled = f(newidx)
    newname = f'{series.name}-{newsamplerate}Hz'
    newseries = pd.Series(resampled, index=newidx, name=newname)
    return (newseries, newsamplerate)


def dopplercut(series, samplerate, parameters):
    (minvel,) = parameters
    (notlow,) = (series < minvel).to_numpy().nonzero()
    newname = f'{series.name}-{minvel}+'
    newseries = series.rename(newname)
    newseries.iloc[notlow] = 0
    return (newseries, None)


def integrate(series, samplerate, parameters):
    (duration,) = parameters
    window = int(np.floor(duration * samplerate))
    integrated = series.rolling(window, center=True).sum()
    newname = f'{series.name}-integrate{duration}s'
    newseries = integrated.rename(newname)
    return (newseries, None)


def diff(series, samplerate, parameters):
    (order, timeunit) = parameters
    timeunit = timeunit.strip()
    dy = np.diff(series.values, order)
    dt = np.diff(series.index.values, order)
    ureg = UnitRegistry()
    ptu = ureg.parse_expression(timeunit)
    conv = ptu.to(ureg.nanosecond).magnitude
    dt = dt.astype(float) / conv
    diffed = dy / dt
    newname = f'{series.name}-diff{order}(/{timeunit})'
    newseries = pd.Series(diffed, index=series.index[order:], name=newname)
    return (newseries, None)


filtfuncs = {
    'savgol': savgol,
    'lag': lag,
    'tf': tf,
    'sma': sma,
    'lowpass': lowpass,
    'ventilation': ventilation,
    'interp': interp,
    'dopplercut': dopplercut,
    'integrate': integrate,
    'diff': diff,
    'norm1': norm1,
    'norm2': norm2,
    'expression': expression,
    'setdatetime': setdatetime,
}


def filter(curve, filtname, paramgetter):
    samplerate = curve.samplerate
    series = curve.series
    filt = Filters[filtname]
    parameters = map(paramgetter, filt.parameters)
    return filtfuncs[filt.name](series, samplerate, parameters)


def filterFeet(starts, stops, filtname, paramgetter):
    filt = FeetFilters[filtname]
    parameters = map(paramgetter, filt.parameters)

    if filt.name == 'shortcycles':
        if len(stops) < 1:
            # No stop information
            raise ValueError('No stop feet')
        (minDuration,) = parameters
        minCycleLength = minDuration * 1e9  # Transform ms to ns
        cycleDurations = (stop - start for start, stop in zip(starts, stops))
        boolidx = [d >= minCycleLength for d in cycleDurations]
        starts, stops, boolidx = truncatevecs([starts, stops, boolidx])
        newstarts = starts[boolidx]
        newstops = stops[boolidx]
    else:
        errmsg = f'Unknown filter: {filtname}'
        raise ValueError(errmsg)
    return (newstarts, newstops)



from .csv import curves_to_csv
from .edf import curves_to_edf
from .matlab import curves_to_matlab
from .parquet import export_curves

curve_writers = {
    'csv': curves_to_csv,
    'edf': curves_to_edf,
    'parquet': export_curves,
    'mat': curves_to_matlab,
}

curve_writers = {k: f for k, f in curve_writers.items() if f.is_available}



import string
from functools import partial

import numexpr as ne
import pandas as pd
from graphysio import dialogs, transformations
from graphysio.algorithms import filters
from graphysio.plotwidgets import (
    LoopWidget,
    PlotWidget,
    POISelectorWidget,
    SpectrogramWidget,
)
from graphysio.structures import CycleId, Parameter, PlotData
from graphysio.writedata import exporter
from pyqtgraph.Qt import QtCore, QtGui


class TSWidget(PlotWidget):
    def __init__(self, plotdata, parent=None, properties=None):
        if properties is None:
            properties = {}
        super().__init__(plotdata.name, parent=parent, properties=properties)
        self.exporter = exporter.TsExporter(self, plotdata.name)
        self.appendData(plotdata)

    def filterCurve(self, oldcurve, filtername, asnew=False):
        newseries, newsamplerate = filters.filter(
            oldcurve, filtername, dialogs.askUserValue
        )
        if asnew:
            newname = self.validateNewCurveName(newseries.name)
            if newname != newseries.name:
                newseries = newseries.rename(newname)
            newcurve = self.addSeriesAsCurve(series=newseries)
            if newsamplerate and newcurve:
                newcurve.set_samplerate(newsamplerate)
        else:
            newseries = newseries.rename(oldcurve.series.name)
            oldcurve.replace_data(newseries)
            if newsamplerate:
                oldcurve.set_samplerate(newsamplerate)

    def filterFeet(self, curve, filtername, asnew=False):
        feetdict = curve.feetitem.indices
        oldstarts = feetdict['start']
        oldstops = feetdict['stop']
        starts, stops = filters.filterFeet(
            oldstarts, oldstops, filtername, dialogs.askUserValue
        )
        feetdict['start'] = starts
        feetdict['stop'] = stops
        curve.feetitem.render()

    def keyPressEvent(self, event):
        if event.key() == QtCore.Qt.Key_Delete:
            for curve in self.curves.values():
                curve.feetitem.removeSelection()

    # Menu Curves
    def showCurveList(self):
        dlgCurveSelection = dialogs.DlgCurveSelection(
            parent=self.parent,
            visible=self.curves.values(),
            hidden=self.hiddencurves,
        )

        def cb(result):
            currently_visible = set(self.curves.values())
            currently_hidden = self.hiddencurves
            allcurves = currently_visible | currently_hidden
            newly_visible, curveproperties = result
            newly_hidden = allcurves - newly_visible

            for c in curveproperties:
                self.applyCurveProperties(c, curveproperties[c])

            for c in newly_visible:
                self.addCurve(c)

            for c in newly_hidden:
                self.removeCurve(c)

            self.hiddencurves = newly_hidden
            self.rebuildLegend()

        dlgCurveSelection.dlgdata.connect(cb)
        dlgCurveSelection.exec_()

    def launchCycleDetection(self):
        dlgCycles = dialogs.DlgCycleDetection(parent=self.parent)

        def cb(choices):
            for curvename, choice in choices.items():
                curve = self.curves[curvename]
                curve.addFeet(CycleId(choice))

        dlgCycles.dlgdata.connect(cb)
        dlgCycles.exec_()

    def launchFilter(self, filterfeet):
        dlgFilter = dialogs.DlgFilter(parent=self.parent, filterfeet=filterfeet)

        def cb(result):
            createnew, curvechoices = result
            filterfunc = self.filterFeet if filterfeet else self.filterCurve
            for curvename, choice in curvechoices.items():
                if choice == 'None':
                    continue
                curve = self.curves[curvename]
                filterfunc(curve, choice, asnew=createnew)

        dlgFilter.dlgdata.connect(cb)
        dlgFilter.exec_()

    def setDateTime(self):
        if len(self.curves) < 1:
            return
        sortedcurves = sorted(
            self.curves.values(), key=lambda curve: curve.series.index[0]
        )
        fstcurve = sortedcurves[0]
        curtimestamp = fstcurve.series.index[0]
        dlg = dialogs.DlgSetDateTime(prevdatetime=curtimestamp)
        dlg.exec_()
        newtimestamp = dlg.dlgdata
        if not newtimestamp:
            return
        offset = newtimestamp - curtimestamp
        for curve in self.curves.values():
            curve.series.index += offset
            curve.render()

    def launchTransformation(self):
        param = Parameter(
            "Choose Transformation",
            list(transformations.Transformations.keys()),
        )
        qresult = dialogs.askUserValue(param)
        if qresult is None:
            return
        trans = transformations.Transformations[qresult]
        for curve in trans(self):
            self.addCurve(curve)

    # Menu Plot
    def launchPOIWidget(self):
        curvenames = list(self.curves.keys())
        q = Parameter('Select Curve', curvenames)
        curvename = dialogs.askUserValue(q)
        if not curvename:
            return
        curve = self.curves[curvename]
        poiselector = POISelectorWidget(
            curve.series, parent=self.parent, properties=self.properties
        )
        self.parent.addTab(poiselector, curve.name())

    def launchSpectrogram(self):
        curvenames = list(self.curves.keys())
        q = Parameter('Select Curve', curvenames)
        curvename = dialogs.askUserValue(q)
        q = Parameter('Enter time window', 'time')
        window = dialogs.askUserValue(q)
        if not curvename:
            return
        curve = self.curves[curvename]
        spectro = SpectrogramWidget(
            curve.series, curve.samplerate, window, parent=self.parent
        )
        self.parent.addTab(spectro, curve.name())

    def launchCurveAlgebra(self):
        curvecorr = {n: c for n, c in zip(string.ascii_lowercase, self.curves.keys())}
        dlgCurveAlgebra = dialogs.DlgCurveAlgebra(self, curvecorr)

        def cb(formula):
            symbols = list(ne.NumExpr(formula).input_names)
            curvenames = [curvecorr[x] for x in symbols]
            sers = [self.curves[c].series for c in curvenames]
            if not len(sers):
                return

            argsdf = pd.concat(sers, axis=1, keys=symbols)
            argsdf = argsdf.interpolate()
            args = argsdf.to_dict(orient='series')

            newvals = ne.evaluate(formula, local_dict=args)
            newname = self.validateNewCurveName(formula, True)
            newseries = pd.Series(
                newvals, index=argsdf.index, name=newname, dtype='float64'
            )
            self.addSeriesAsCurve(newseries)

        dlgCurveAlgebra.dlgdata.connect(cb)
        dlgCurveAlgebra.exec_()

    # Menu Selection
    def launchNewPlotFromSelection(self):
        xmin, xmax = self.vbrange
        sers = []
        for c in self.curves.values():
            series = c.series.loc[xmin:xmax]
            sers.append(series)
        df = pd.concat(sers, axis=1, keys=[s.name for s in sers])
        newname = f'{self.name}-sub'
        plotdata = PlotData(data=df, name=newname)
        self.parent.createNewPlotWithData(plotdata)

    def launchAppendToPlotFromSelection(self):
        tabWidget = self.parent.tabWidget
        ntabs = tabWidget.count()
        tabdict = {tabWidget.tabText(idx): idx for idx in range(ntabs)}
        desttabname, ok = QtGui.QInputDialog.getItem(
            self,
            'Select destination',
            'Destination plot',
            list(tabdict.keys()),
            editable=False,
        )
        if not ok:
            return
        xmin, xmax = self.vbrange
        sers = []
        for c in self.curves.values():
            series = c.series.loc[xmin:xmax]
            sers.append(series)
        df = pd.concat(sers, axis=1, keys=[s.name for s in sers])
        newname = f'{self.name}-sub'
        plotdata = PlotData(data=df, name=newname)
        self.parent.appendToPlotWithData(plotdata, destidx=tabdict[desttabname])

    def launchLoop(self):
        dlgSetupPU = dialogs.DlgSetupPULoop(self, parent=self.parent)

        def cb(result):
            uname, pname = result
            u = self.curves[uname]
            p = self.curves[pname]
            loopwidget = LoopWidget(u, p, self.vbrange, parent=self.parent)
            newname = f'{self.name}-{p.name()}-loops'
            self.parent.addTab(loopwidget, newname)

        dlgSetupPU.dlgdata.connect(cb)
        dlgSetupPU.exec_()

    @property
    def menu(self):
        mcurves = {
            'Visible Curves': self.showCurveList,
            'Cycle Detection': self.launchCycleDetection,
            'Filter Curve': partial(self.launchFilter, filterfeet=False),
            'Filter Feet': partial(self.launchFilter, filterfeet=True),
            'Transformation': self.launchTransformation,
            'Set Date / Time': self.setDateTime,
        }
        mselect = {
            'As new plot': self.launchNewPlotFromSelection,
            'Append to other plot': self.launchAppendToPlotFromSelection,
            'Generate PU-Loops': self.launchLoop,
        }
        mplot = {
            'POI Selector': self.launchPOIWidget,
            'Spectrogram': self.launchSpectrogram,
            'Curve Algebra': self.launchCurveAlgebra,
        }
        mexport = {
            'Curves': self.exporter.curves,
            'Time info': self.exporter.periods,
            'Cycle info': self.exporter.cyclepoints,
            'Cycles to directory': self.exporter.cycles,
        }
        return {
            'Curves': mcurves,
            'Plot': mplot,
            'Seletion': mselect,
            'Export': mexport,
        }



from datetime import datetime
from typing import List

import numpy as np
from graphysio.dialogs import askUserValue
from graphysio.plotwidgets.curves import CurveItem
from graphysio.structures import Parameter
from scipy import interpolate

try:
    import pyedflib
except ImportError:
    is_available = False
else:
    is_available = True


def interp_series(s, samplerate, start, stop):
    f = interpolate.interp1d(
        s.index, s.values, assume_sorted=True, copy=False, fill_value='extrapolate'
    )
    # Account for periods of missing data.
    duration_s = (stop - start) * 1e-9
    npoints = int(duration_s * samplerate)
    newidx = np.linspace(start, stop, num=npoints)
    return f(newidx)


def curves_to_edf(
    curves: List[CurveItem], filepath: str, index_label: str = 'timens'
) -> None:
    headers = []
    signals = []

    beginns = min(c.series.index[0] for c in curves)
    endns = max(c.series.index[-1] for c in curves)
    begindt = datetime.fromtimestamp(beginns * 1e-9)

    # Use global min / max of values since some viewers need this (edfbrowser)
    physmax = max(c.series.max() for c in curves)
    physmin = min(c.series.min() for c in curves)

    # Ask the user for the physical dimension shared by all curves
    dim = askUserValue(Parameter('Enter physical dimension', str))

    for c in curves:
        s = c.series
        header = {
            'label': c.name(),
            'sample_rate': c.samplerate,
            'physical_max': physmax,
            'physical_min': physmin,
            'digital_max': 32767,
            'digital_min': -32768,
            'transducer': '',
            'prefilter': '',
            'dimension': dim,
        }
        headers.append(header)
        resampled = interp_series(s, c.samplerate, beginns, endns)
        signals.append(resampled)

    edf = pyedflib.EdfWriter(
        str(filepath), len(signals), file_type=pyedflib.FILETYPE_EDFPLUS
    )
    edf.setStartdatetime(begindt)
    edf.setSignalHeaders(headers)
    edf.writeSamples(signals)
    edf.close()


curves_to_edf.is_available = is_available



from functools import partial
from typing import Optional

import numpy as np
import pandas as pd
import pyqtgraph as pg
from graphysio import utils
from graphysio.algorithms import waveform
from graphysio.structures import CycleId
from graphysio.utils import estimateSampleRate
from physiocurve.pandas.ecg import Ecg
from pyqtgraph.Qt import QtCore, QtGui


class CurveItem(pg.PlotDataItem):
    visible = QtCore.pyqtSignal()
    invisible = QtCore.pyqtSignal()

    def __init__(self, series, parent, pen=None):
        self.parent = parent
        self.series = self.sanitize_data(series)
        if self.series is None:
            raise ValueError('Not enough data')
        self.samplerate = estimateSampleRate(self.series)
        if pen is None:
            pen = QtGui.QColor(QtCore.Qt.black)
        super().__init__(name=series.name, pen=pen, antialias=True)
        self.render()

    def replace_data(self, newdata):
        newdata = self.sanitize_data(newdata)
        if newdata is None:
            # XXX report error
            return
        self.clear()
        self.series = newdata.rename(self.opts['name'])
        self.samplerate = estimateSampleRate(self.series)
        self.render()

    def extend(self, newseries):
        merged = self.series.append(newseries)
        newdata = self.sanitize_data(merged)
        if newdata is None:
            # XXX report error
            return
        self.series = newdata
        self.render()

    def render(self):
        self.setData(x=self.series.index.to_numpy(), y=self.series.to_numpy())

    def set_samplerate(self, newsamplerate):
        self.samplerate = newsamplerate

    def rename(self, newname: str):
        self.opts['name'] = newname
        self.series = self.series.rename(newname)

    def sanitize_data(self, series: pd.Series) -> Optional[pd.Series]:
        series = series.dropna()
        series = series.sort_index()
        # Less than 2 points are not visible
        if len(series) < 2:
            return None
        # Make timestamp unique and use mean of values on duplicates
        series = series.groupby(level=0).mean()
        return series


class POIItem(pg.ScatterPlotItem):
    sym = {
        'start': 'star',
        'stop': 's',
        'diastole': 't1',
        'systole': 't',
        'dicrotic': 'd',
        'point': 'o',
        'rwave': '+',
    }
    # Symbols = OrderedDict([(name, QtGui.QPainterPath()) for name in
    # ['o', 's', 't', 't1', 't2', 't3','d', '+', 'x', 'p', 'h', 'star']])

    def __init__(self, parent, name, pen=None):
        super().__init__(pen=pen, name=name)
        self.parent = parent
        self.indices = {}
        self.selected = []
        self.resym = {value: key for key, value in self.sym.items()}
        self.render()

    def addPointsByLocation(self, key, locations):
        if key not in self.indices:
            self.indices[key] = pd.Index([])
        oldidx = self.indices[key]
        newidx = oldidx.append(pd.Index(locations))
        self.indices[key] = newidx.unique().sort_values()
        self.render()

    def removePointsByLocation(self, key, locations):
        if key not in self.indices:
            return
        oldidx = self.indices[key]
        dellocs = []
        for loc in locations:
            locidx = oldidx.get_loc(loc, method='nearest')
            dellocs.append(locidx)
        newidx = oldidx.delete(dellocs)
        self.indices[key] = newidx
        self.render()

    def removePoints(self, points):
        for point in points:
            try:
                sym = self.resym[point.symbol()]
                idx = self.indices[sym]
            except KeyError:
                # Should not happen
                continue
            nidx = idx.get_loc(point.pos().x(), method='nearest')
            self.indices[sym] = idx.delete(nidx)
        self.render()

    def render(self):
        data = []
        for key, idx in self.indices.items():
            if len(idx) < 1:
                continue
            idxnona = idx.dropna()
            points = self.parent.series.loc[idxnona]
            tmp = pd.DataFrame({'points': points, 'sym': self.sym[key]}, index=idxnona)
            data.append(tmp)
        if len(data) < 1:
            self.clear()
            return
        feet = pd.concat(data)
        self.setData(
            x=feet.index.values, y=feet['points'].values, symbol=feet['sym'].values
        )

    def isPointSelected(self, point):
        return point in self.selected

    def selectPoint(self, point):
        if not self.isPointSelected(point):
            self.selected.append(point)
        point.setPen('r')
        point.setBrush('r')

    def unselectPoint(self, point):
        if self.isPointSelected(point):
            self.selected.remove(point)
        point.resetPen()
        point.resetBrush()

    def removeSelection(self):
        self.removePoints(self.selected)
        self.selected = []
        self.render()

    def rename(self, newname: str):
        self.opts['name'] = newname


class CurveItemWithPOI(CurveItem):
    visible = QtCore.pyqtSignal()
    invisible = QtCore.pyqtSignal()

    @staticmethod
    def sigPointClicked(feetitem, points):
        point = points[0]  # only one point per click
        if not feetitem.isPointSelected(point):
            feetitem.selectPoint(point)
        else:
            feetitem.unselectPoint(point)

    def __init__(self, series, parent, pen=None):
        super().__init__(series, parent, pen)
        feetname = f'{series.name}-feet'
        self.feetitem = POIItem(self, name=feetname, pen=pen)
        parent.addItem(self.feetitem)
        self.feetitem.sigClicked.connect(self.sigPointClicked)
        self.visible.connect(self.__becameVisible)
        self.invisible.connect(self.__becameInvisible)

    def __becameVisible(self):
        if self.feetitem not in self.parent.listDataItems():
            self.parent.addItem(self.feetitem)
        self.render()
        self.feetitem.render()

    def __becameInvisible(self):
        self.parent.removeItem(self.feetitem)

    def addFeet(self, cycleid):
        if cycleid is CycleId.none:
            return
        elif cycleid is CycleId.velocity:
            starts, stops = waveform.findFlowCycles(self)
            self.feetitem.indices['start'] = starts
            self.feetitem.indices['stop'] = stops
        elif cycleid is CycleId.foot:
            foot = waveform.findPressureFeet(self)
            self.feetitem.indices['start'] = foot
        elif cycleid is CycleId.pressure:
            if 'start' not in self.feetitem.indices:
                self.addFeet(CycleId.foot)
            dia, sbp, dic = waveform.findPressureFull(self)
            self.feetitem.indices['diastole'] = dia
            self.feetitem.indices['systole'] = sbp
            self.feetitem.indices['dicrotic'] = dic
        elif cycleid is CycleId.rwave:
            ecg = Ecg(self.series)
            self.feetitem.indices['rwave'] = pd.Index(ecg.idxrwave)
        else:
            raise ValueError(cycleid)
        self.feetitem.render()

    def getCycleIndices(self, vrange=None):
        s = self.series
        clip = partial(utils.clip, vrange=vrange)
        hasstarts = ('start' in self.feetitem.indices) and self.feetitem.indices[
            'start'
        ].size > 0
        hasstops = ('stop' in self.feetitem.indices) and self.feetitem.indices[
            'stop'
        ].size > 0
        if vrange:
            xmin, xmax = vrange
        else:
            xmin = s.index[0]
            xmax = s.index[-1]
        if not hasstarts:
            # We have no feet, treat the whole signal as one cycle
            locs = (s.index.get_loc(i, method='nearest') for i in [xmin, xmax])
            indices = (s.index[loc] for loc in locs)
            begins, ends = [np.array([i]) for i in indices]
        elif not hasstops:
            # We have no stops, starts serve as stops for previous cycle
            begins = clip(self.feetitem.indices['start'].values)
            endloc = s.index.get_loc(xmax, method='nearest')
            end = s.index[endloc]
            ends = np.append(begins[1:], end)
        else:
            # We have starts and stops, use them
            begins = self.feetitem.indices['start'].values
            ends = self.feetitem.indices['stop'].values
            begins, ends = map(clip, [begins, ends])

        # Handle the case where we start in the middle of a cycle
        while ends[0] <= begins[0]:
            ends = ends[1:]

        begins, ends = utils.truncatevecs([begins, ends])
        durations = ends - begins
        return (begins, durations)

    def getFeetPoints(self, feetname):
        feetidx = self.feetitem.indices[feetname]
        feetnona = feetidx[pd.notnull(feetidx)]
        return self.series.loc[feetnona]



from graphysio.structures import PlotData


class BaseReader:
    def __init__(self):
        self.userdata = {}

    def set_data(self, data):
        self.userdata.update(data)

    def askUserInput(self):
        pass

    def __call__(self) -> PlotData:
        raise NotImplementedError



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'filter.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Filter(object):
    def setupUi(self, Filter):
        Filter.setObjectName("Filter")
        Filter.resize(387, 371)
        self.verticalLayout = QtWidgets.QVBoxLayout(Filter)
        self.verticalLayout.setObjectName("verticalLayout")
        self.table = QtWidgets.QTableWidget(Filter)
        self.table.setColumnCount(2)
        self.table.setObjectName("table")
        self.table.setRowCount(0)
        self.verticalLayout.addWidget(self.table)
        self.chkNewcurve = QtWidgets.QCheckBox(Filter)
        self.chkNewcurve.setObjectName("chkNewcurve")
        self.verticalLayout.addWidget(self.chkNewcurve)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.okButton = QtWidgets.QPushButton(Filter)
        self.okButton.setObjectName("okButton")
        self.horizontalLayout.addWidget(self.okButton)
        self.cancelButton = QtWidgets.QPushButton(Filter)
        self.cancelButton.setObjectName("cancelButton")
        self.horizontalLayout.addWidget(self.cancelButton)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(Filter)
        QtCore.QMetaObject.connectSlotsByName(Filter)

    def retranslateUi(self, Filter):
        _translate = QtCore.QCoreApplication.translate
        Filter.setWindowTitle(_translate("Filter", "Filter"))
        self.chkNewcurve.setText(_translate("Filter", "Create new curve"))
        self.okButton.setText(_translate("Filter", "Ok"))
        self.cancelButton.setText(_translate("Filter", "Cancel"))



import csv
from dataclasses import dataclass
from functools import partial
from pathlib import Path
from typing import List

import numpy as np
import pandas as pd
from graphysio import ui
from graphysio.readdata.baseclass import BaseReader
from graphysio.structures import PlotData
from pyqtgraph.Qt import QtCore, QtGui, QtWidgets


class CsvReader(BaseReader):
    is_available = True

    def askUserInput(self):
        filepath = self.userdata['filepath']
        dlg = DlgNewPlotCsv(filepath)
        dlg.exec_()
        csvrequest = dlg.csvrequest
        if csvrequest:
            self.userdata['csvrequest'] = dlg.csvrequest

    def __call__(self) -> List[PlotData]:
        request = self.userdata['csvrequest']
        data = pd.read_csv(
            request.filepath,
            sep=request.seperator,
            usecols=request.fields,
            decimal=request.decimal,
            skiprows=request.droplines,
            encoding=request.encoding,
            index_col=False,
            engine='c',
        )
        pdtonum = partial(pd.to_numeric, errors='coerce')
        dtformat = request.datetime_format
        if request.generatex:
            data.index = (1e9 * data.index / request.samplerate).astype(np.int64)
            # Make all data numeric and remove empty rows
            data = data.apply(pdtonum)
            data = data.dropna(axis='rows', how='all')
        else:
            timestamp = data[request.dtfield]
            data = data.drop(columns=request.dtfield)
            # Force all columns to numeric
            data = data.apply(pdtonum)

            if dtformat == '<seconds>':
                timestamp = pdtonum(timestamp)
                timestamp = pd.to_datetime(timestamp * 1e9, unit='ns')
            elif dtformat == '<milliseconds>':
                timestamp = pdtonum(timestamp)
                timestamp = pd.to_datetime(timestamp * 1e6, unit='ns')
            elif dtformat == '<microseconds>':
                timestamp = pdtonum(timestamp)
                timestamp = pd.to_datetime(timestamp * 1e3, unit='ns')
            elif dtformat == '<nanoseconds>':
                timestamp = pdtonum(timestamp)
                timestamp = pd.to_datetime(timestamp, unit='ns')
            elif dtformat == '<infer>':
                timestamp = pd.to_datetime(timestamp, infer_datetime_format=True)
            else:
                timestamp = pd.to_datetime(timestamp, format=dtformat)
                timestamp = pd.Index(timestamp)
                if timestamp.tz is None:
                    timestamp = timestamp.tz_localize(request.timezone)
                timestamp = timestamp.tz_convert('UTC')

            timestamp = timestamp.view(np.int64)
            data = data.set_index([timestamp])

        data = data.dropna(axis='columns', how='all')
        data = data.sort_index()

        fp = request.filepath
        if request.clusterid:
            g = data.groupby(request.clusterid)
            plotdata = (
                PlotData(
                    data=df.drop(columns=request.clusterid),
                    filepath=fp,
                    name=f'{fp.stem}-{i}',
                )
                for i, df in g
            )
        else:
            plotdata = [PlotData(data=data, filepath=fp)]

        return plotdata


@dataclass
class CsvRequest:
    """Group needed parameters to parse the CSV file."""

    filepath: Path
    seperator: str
    decimal: str
    dtfield: str
    yfields: List[str]
    datetime_format: str
    droplines: int
    generatex: bool
    clusterid: str
    timezone: str
    encoding: str
    samplerate: int

    @property
    def fields(self) -> List[str]:
        dtfields = [] if self.dtfield is None else [self.dtfield]
        clusterfields = [] if self.clusterid is None else [self.clusterid]
        return dtfields + clusterfields + self.yfields


class DlgNewPlotCsv(ui.Ui_NewPlot, QtWidgets.QDialog):
    def __init__(self, filepath, parent=None):
        super().__init__(parent=parent)
        self.setupUi(self)
        self.setWindowTitle(f'Open {filepath.name}')

        self.filepath = filepath
        self.csvrequest = None

        # Attach models to ListViews
        self.lstX = QtGui.QStandardItemModel()
        self.lstY = QtGui.QStandardItemModel()
        self.lstCluster = QtGui.QStandardItemModel()
        self.lstAll = QtGui.QStandardItemModel()

        self.lstVX.setModel(self.lstX)
        self.lstVY.setModel(self.lstY)
        self.lstVCluster.setModel(self.lstCluster)
        self.lstVAll.setModel(self.lstAll)

        # Setup Field Table
        self.lstVAll.verticalHeader().hide()
        self.lstVAll.horizontalHeader().setResizeMode(QtGui.QHeaderView.Stretch)
        self.lstVAll.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)

        # Connect callbacks
        self.btnLoad.clicked.connect(self.loadCsvFields)
        self.btnOk.clicked.connect(self.loadPlot)
        self.btnCancel.clicked.connect(self.reject)
        self.btnToX.clicked.connect(self.moveToX)
        self.btnToY.clicked.connect(self.moveToY)
        self.btnToCluster.clicked.connect(self.moveToCluster)
        self.btnRemoveX.clicked.connect(self.delFromX)
        self.btnRemoveY.clicked.connect(self.delFromY)
        self.btnRemoveCluster.clicked.connect(self.delFromCluster)
        self.lstVX.currentIndexChanged.connect(self.xChanged)

        # Guesstimate CSV field and decimal seperators
        delims = self.estimateDelimiters(filepath)
        self.txtSep.setEditText(delims[0])
        self.txtDecimal.setEditText(delims[1])
        self.txtDateTime.setEditText(f'%Y-%m-%d %H:%M:%S{delims[1]}%f')

    # Methods / Callbacks
    def estimateDelimiters(self, filepath):
        encoding = self.txtEncoding.currentText()
        with open(filepath, 'r', encoding=encoding) as csvfile:
            seperator = ';' if ';' in next(csvfile) else ','
            decimal = '.' if '.' in next(csvfile) else ','
        return (seperator, decimal)

    def loadCsvFields(self):
        sep = self.txtSep.currentText()
        if sep == '<tab>':
            sep = '\t'
        # Use the csv module to retrieve csv fields
        for lst in [self.lstAll, self.lstX, self.lstY]:
            lst.clear()
        self.lstAll.setHorizontalHeaderLabels(["Field", "1st Line"])
        encoding = self.txtEncoding.currentText()
        with open(self.filepath, 'r', encoding=encoding) as csvfile:
            # Artificially drop n first lines as requested
            for _ in range(self.spnLinedrop.value()):
                next(csvfile)
            csvreader = csv.DictReader(csvfile, delimiter=sep)
            row = next(csvreader)
            for key, value in row.items():
                if key is None:
                    continue
                keyitem = QtGui.QStandardItem(key)
                valueitem = QtGui.QStandardItem(value)
                self.lstAll.appendRow([keyitem, valueitem])
        self.lstAll.sort(0)

    def xChanged(self, _newtext):
        if self.lstX.rowCount() > 0:
            self.chkGenX.setCheckState(QtCore.Qt.Unchecked)
        else:
            self.chkGenX.setCheckState(QtCore.Qt.Checked)

    def moveToCluster(self):
        if self.lstCluster.rowCount() > 0:
            # Only allow one element for Cluster Id.
            return
        selection = self.lstVAll.selectedIndexes()
        rowindex = selection[0].row()
        row = self.lstAll.takeRow(rowindex)
        self.lstCluster.appendRow(row)

    def moveToX(self):
        if self.lstX.rowCount() > 0:
            # Only allow one element for X.
            return
        selection = self.lstVAll.selectedIndexes()
        rowindex = selection[0].row()
        row = self.lstAll.takeRow(rowindex)
        self.lstX.appendRow(row)

    def moveToY(self):
        while True:
            selection = self.lstVAll.selectedIndexes()
            if len(selection) < 1:
                break
            rowindex = selection[0].row()
            self.lstY.appendRow(self.lstAll.takeRow(rowindex))

    def delFromCluster(self):
        if not self.lstCluster.rowCount():
            return
        row = self.lstCluster.takeRow(0)
        self.lstAll.appendRow(row)

    def delFromX(self):
        if not self.lstX.rowCount():
            return
        row = self.lstX.takeRow(0)
        self.lstAll.appendRow(row)

    def delFromY(self):
        while True:
            rowindexes = self.lstVY.selectedIndexes()
            if len(rowindexes) < 1:
                break
            row = rowindexes[0].row()
            self.lstAll.appendRow(self.lstY.takeRow(row))

    def loadPlot(self):
        yRows = [i.text() for i in self.lstY.findItems("", QtCore.Qt.MatchContains)]
        xRows = [i.text() for i in self.lstX.findItems("", QtCore.Qt.MatchContains)]
        cRows = [
            i.text() for i in self.lstCluster.findItems("", QtCore.Qt.MatchContains)
        ]

        seperator = self.txtSep.currentText()
        seperator = '\t' if seperator == '<tab>' else seperator

        generatex = self.chkGenX.checkState() > QtCore.Qt.Unchecked
        try:
            dtfield = xRows[0]
        except IndexError:
            dtfield = None

        try:
            cid = cRows[0]
        except IndexError:
            cid = None

        req = CsvRequest(
            filepath=self.filepath,
            seperator=seperator,
            decimal=self.txtDecimal.currentText(),
            dtfield=dtfield,
            yfields=yRows,
            datetime_format=self.txtDateTime.currentText(),
            droplines=self.spnLinedrop.value(),
            generatex=generatex,
            clusterid=cid,
            timezone=self.txtTimezone.currentText(),
            encoding=self.txtEncoding.currentText(),
            samplerate=self.spnFs.value(),
        )

        self.csvrequest = req
        self.accept()



from datetime import timedelta
from functools import partial
from typing import Optional

import pyqtgraph as pg
from graphysio.legend import LegendItem
from graphysio.plotwidgets.curves import CurveItem, CurveItemWithPOI
from graphysio.utils import Colors
from pyqtgraph.Qt import QtCore, QtGui


class TimeAxisItem(pg.AxisItem):
    @staticmethod
    def conv_absolute(value, mainwindow=False):
        value = int(value * 1e-6)  # convert from ns to ms
        date = QtCore.QDateTime.fromMSecsSinceEpoch(value)
        date = date.toTimeSpec(QtCore.Qt.UTC)
        if mainwindow:
            timestr = date.toString("dd/MM/yyyy hh:mm:ss.zzz")
        else:
            timestr = date.toString("dd/MM/yyyy\nhh:mm:ss.zzz")
        return timestr

    @staticmethod
    def conv_relative(value):
        value = int(value * 1e-6)  # convert from ns to ms
        td = timedelta(milliseconds=value)
        return str(td)

    @staticmethod
    def is_relative_time(val):
        return val < 5e14  # Nov 1985

    def tickStrings(self, values, _scale, _spacing):
        if not values:
            return []
        if self.is_relative_time(values[0]):
            conv = self.conv_relative
        else:
            conv = self.conv_absolute
        return [conv(v) for v in values]


class PlotWidget(pg.PlotWidget):
    @staticmethod
    def mouseMoved(self, evt):
        pos = evt[0]  # using signal proxy turns original arguments into a tuple
        if self.sceneBoundingRect().contains(pos):
            mousePoint = self.getViewBox().mapSceneToView(pos)
            self.parent.setcoords.emit(mousePoint.x(), mousePoint.y())

    def __init__(self, name, parent=None, properties=None):
        self.parent = parent
        self.name = name
        self.colors = Colors()
        self.hiddencurves = set()
        self.properties = properties if properties is not None else {}

        axisItems = {'bottom': TimeAxisItem(orientation='bottom')}
        super().__init__(parent=parent, axisItems=axisItems, background='w')

        self.legend = LegendItem()
        self.legend.setParentItem(self.getPlotItem())

        self.vb = self.getViewBox()
        self.vb.setMouseMode(self.vb.RectMode)

        self.setCursor(QtCore.Qt.CrossCursor)

        mouseMoved = partial(self.mouseMoved, self)
        self.sigproxy = pg.SignalProxy(
            self.scene().sigMouseMoved, rateLimit=60, slot=mouseMoved
        )

    def appendData(self, newplotdata, dorealign=False):
        for seriesname in newplotdata.data:
            self.addSeriesAsCurve(newplotdata.data[seriesname], dorealign=dorealign)

    @property
    def curves(self):
        return {
            item.name(): item
            for item in self.listDataItems()
            if isinstance(item, CurveItem)
        }

    def getPen(self):
        return next(self.colors)

    def addSeriesAsCurve(
        self, series, pen=None, dorealign=False, withfeet=True
    ) -> Optional[CurveItem]:
        if dorealign and self.curves:
            # Timeshift new curves to make the beginnings coincide
            begins = [curve.series.index[0] for curve in self.curves.values()]
            offset = min(begins) - series.index[0]
            series.index += offset
        try:
            # Append to existing curve?
            curve = self.curves[series.name]
            curve.extend(series)
        except KeyError:
            # New curve
            if pen is None:
                pen = self.getPen()
            Curve = CurveItemWithPOI if withfeet else CurveItem
            try:
                curve = Curve(series=series, parent=self, pen=pen)
            except ValueError:
                return None
            self.addCurve(curve)
        return curve

    def addCurve(self, curve, pen=None):
        if curve.name() in self.curves:
            return
        if pen is not None:
            curve.setPen(pen)
        self.addItem(curve)
        self.rebuildLegend()
        curve.visible.emit()

    def removeCurve(self, curve):
        if curve.name() not in self.curves:
            return
        self.removeItem(curve)
        self.rebuildLegend()
        curve.invisible.emit()

    def rebuildLegend(self):
        self.legend.clear()
        for _, curve in self.curves.items():
            if isinstance(curve, pg.PlotDataItem):
                self.legend.addItem(curve, curve.name())

    def validateNewCurveName(
        self, proposedname: str, alwaysShow: bool = False
    ) -> Optional[str]:
        if proposedname not in self.curves and not alwaysShow:
            return proposedname
        newname, okPressed = QtGui.QInputDialog.getText(
            self,
            "Series name",
            "Series with identical names will be merged.",
            text=proposedname,
        )
        if not okPressed:
            return None
        return newname

    def applyCurveProperties(self, curve, properties: dict) -> None:
        # Setting Symbol and Connect does not work. TODO dig into pyqtgraph
        # curve.setData(symbol=properties['symbol'])
        # curve.setData(connect=properties['connect'], symbol=properties['symbol'])
        curve.setPen(properties['color'], width=properties['width'])
        curve.rename(properties['name'])

    @property
    def vbrange(self):
        vbrange = self.vb.viewRange()
        xmin, xmax = map(int, vbrange[0])
        return (xmin, xmax)

    @property
    def menu(self):
        return {}



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['graphysio',
 'graphysio.algorithms',
 'graphysio.plotwidgets',
 'graphysio.readdata',
 'graphysio.ui',
 'graphysio.writedata']

package_data = \
{'': ['*']}

install_requires = \
['Pint>=0.19.2,<0.20.0',
 'numexpr>=2.8.0,<3.0.0',
 'pandas>=1.4.3,<2.0.0',
 'pathvalidate>=2.5.0,<3.0.0',
 'physiocurve>=2022.6.26,<2023.0.0',
 'pyqtgraph>=0.12.4,<0.13.0',
 'scipy>=1.8.1,<2.0.0']

entry_points = \
{'console_scripts': ['graphysio = graphysio.main:main']}

setup_kwargs = {
    'name': 'graphysio',
    'version': '2022.7.7',
    'description': 'Visualizer and analyser for biometric signals',
    'long_description': '# GraPhysio\nGraPhysio is a graphical time series visualizer created for biometric data\nsignals from ICU patient monitors. It is however not limited to this. It can\nhandle low frequency and high frequency data as well as aggregating and\nsynchronizing signals from different sources. GraPhysio supports basic\nmathematical operations and filters and can help selecting and exporting time\nperiods. GraPhysio can read data from CSV, Parquet and EDF files and can write\nCSV, Parquet, EDF and Matlab files.\n\n## Install instructions\nFor the best experience, conda is recommended:\n\nconda install -c conda-forge graphysio\n\nAlternatively you can then install the latest version of GraPhysio from PyPi by\ntying the following command:\n\n> python -m pip install graphysio\n\nYou can launch GraPhysio by typing:\n\n> python -m graphysio\n',
    'author': 'Jona Joachim',
    'author_email': 'jona@joachim.cc',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jaj42/GraPhysio',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<3.11',
}


setup(**setup_kwargs)



from graphysio.plotwidgets.plotwidget import PlotWidget, TimeAxisItem
from graphysio.plotwidgets.poiselector import POISelectorWidget
from graphysio.plotwidgets.puplot import LoopWidget
from graphysio.plotwidgets.spectrogram import SpectrogramWidget
from graphysio.plotwidgets.tsplot import TSWidget

__all__ = [
    'PlotWidget',
    'TimeAxisItem',
    'POISelectorWidget',
    'LoopWidget',
    'SpectrogramWidget',
    'TSWidget',
]



from collections import namedtuple
from enum import Enum
from pathlib import Path
from typing import Optional

Filter = namedtuple('Filter', ['name', 'parameters'])
Parameter = namedtuple('Parameter', ['description', 'request'])


class CycleId(Enum):
    none = 'None'
    foot = 'Pressure foot'
    pressure = 'Pressure Full'
    velocity = 'Velocity'
    rwave = 'ECG R Wave'


class PlotData:
    def __init__(self, data, filepath: Path, name: Optional[str] = None):
        self.data = data
        self.filepath = Path(filepath)
        self._name = name

    @property
    def name(self):
        if self._name is not None:
            name = self._name
        else:
            name = self.filepath.stem
        return name

    @property
    def folder(self):
        return self.filepath.parent
